import React, { createContext, useContext } from 'react';
import { useAddDropsForm } from '../forms/useAddDropsForm';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';

function useLiquidityPairAddDropsState(props: Parameters<typeof useAddDropsForm>[0]) {
  const form = useAddDropsForm(props);
  return form;
}

export type LiquidityPairAddDropsContextType = ReturnType<typeof useLiquidityPairAddDropsState>;

const LiquidityPairAddDropsContext = createContext<LiquidityPairAddDropsContextType>(null as any);

export function useLiquidityPairAddDrops() {
  const state = useContext(LiquidityPairAddDropsContext);
  if (!state) throw new Error('LiquidityPairAddDropsContext missing');
  return state;
}

// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_LOADING_RENDER_FN() {
  return <PlaceholderLoadingIndicator text="Loading add drops form" />;
}
// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_ERROR_RENDER_FN(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function LiquidityPairAddDropsProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
  ...props
}: {
  children: JSX.Element | JSX.Element[];
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
} & Parameters<typeof useLiquidityPairAddDropsState>[0]) {
  const state = useLiquidityPairAddDropsState(props);

  if (state.loadingStatus.pending) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);
  return (
    <LiquidityPairAddDropsContext.Provider value={state}>
      {children}
    </LiquidityPairAddDropsContext.Provider>
  );
}
