import React, { createContext, useContext } from 'react';
import { useGovVoteForm } from '@onomy/react-hub';

import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';

export type ProposalVoteContextType = ReturnType<typeof useGovVoteForm>;

export const ProposalVoteContext = createContext<ProposalVoteContextType>(null as any);

export function useProposalVote() {
  const state = useContext(ProposalVoteContext);
  if (!state) throw new Error('Missing ProposalVoteContext');
  return state;
}

function defaultRenderError(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function ProposalVoteProvider({
  children,
  renderLoading,
  renderError = defaultRenderError,
  ...props
}: {
  children: JSX.Element | JSX.Element[];
  proposalId: string;
  renderLoading?: () => JSX.Element | JSX.Element[] | null;
  renderError?: typeof defaultRenderError;
} & Parameters<typeof useGovVoteForm>[0]) {
  const state = useGovVoteForm(props);

  if (renderLoading && (state.loadingStatus.pending || !state.proposal)) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);

  return <ProposalVoteContext.Provider value={state}>{children}</ProposalVoteContext.Provider>;
}
