import React, { createContext, useContext } from 'react';
import { useClaimRewardForm } from '@onomy/react-hub';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';

export type ClaimRewardContextType = ReturnType<typeof useClaimRewardForm>;

const ClaimRewardContext = createContext<ClaimRewardContextType>(null as any);

export function useClaimReward() {
  const state = useContext(ClaimRewardContext);
  if (!state) throw new Error('ClaimRewardContext missing');
  return state;
}

function defaultRenderError(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function ClaimRewardProvider({
  children,
  renderLoading,
  renderError = defaultRenderError,
  ...props
}: {
  children: JSX.Element | JSX.Element[];
  validatorId: string;
  redelegate?: boolean;
  renderLoading?: () => JSX.Element | JSX.Element[] | null;
  renderError?: typeof defaultRenderError;
} & Parameters<typeof useClaimRewardForm>[0]) {
  const state = useClaimRewardForm(props);

  if (renderLoading && (state.loadingStatus.pending || !state.delegation)) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);
  return <ClaimRewardContext.Provider value={state}>{children}</ClaimRewardContext.Provider>;
}
